import unittest
from unittest.mock import Mock, patch
from datetime import datetime
from freezegun import freeze_time

from ms_imputedhours_core.employee import get_real_fte, calculate_real_capacity


DEFAULT_AGREEMENT_HOURS = {
    'totalHours': 150.0, 
    'days': {
        1: '0', 
        2: '8', 
        3: '8', 
        4: '5.5', 
        5: '0', 
        6: '0',
        7: '8', 
        8: '8', 
        9: '0', 
        10: '8', 
        11: '5.5', 
        12: '0',
        13: '0', 
        14: '8',
        15: '8',
        16: '8',
        17: '8', 
        18: '5.5',
        19: '0',
        20: '0', 
        21: '8', 
        22: '8', 
        23: '8', 
        24: '8',
        25: '5.5',
        26: '0', 
        27: '0', 
        28: '8',
        29: '8', 
        30: '8'
    }
}

class TestSuite(unittest.TestCase): 
    @freeze_time("2022-11-11")
    def test_get_real_fte_returns_successfactor_fte_when_date_is_not_future(self):
        expected_result = 1.0 ## 1.0 === 100%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 50}
        date = Mock()
        date.year = 2021
        date.month = 10
        
        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)
    
    @freeze_time("2021-11-11")
    def test_get_real_fte_returns_employee_fte_when_date_is_for_future(self):
        expected_result = 0.5 # 50%
        successfactor_data = {'FTE': 100}
        employeeFTE = {'fte': 50}
        date = Mock()
        date.year = 2022
        date.month = 10
        
        result = get_real_fte(successfactor_data, employeeFTE, date)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_full_capacity_when_is_normal_month(self):
        """ 
        A normal month is a definition for a month where the employee 
        will work 100% of the hours of his agreement, and does not 
        coincide with any special date as the end or start 
        of the employment contract.
        """
        expected_result = 150.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 1.0 # 100%
        calculate_range = False
        
        result = calculate_real_capacity(DEFAULT_AGREEMENT_HOURS, hiring_date, end_date, date, to_date, fte, calculate_range)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_half_capacity_when_is_normal_month_and_fte_zero_point_five(self):
        """ 
        A normal month is a definition for a month where the employee 
        will work 100% of the hours of his agreement, and does not 
        coincide with any special date as the end or start 
        of the employment contract.
        """
        expected_result = 75.0
        hiring_date = datetime.strptime('17/09/2021', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5 # 50%
        calculate_range = False
        
        result = calculate_real_capacity(DEFAULT_AGREEMENT_HOURS, hiring_date, end_date, date, to_date, fte, calculate_range)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_zero_when_date_is_before_hired(self):
        """ 
        A normal month is a definition for a month where the employee 
        will work 100% of the hours of his agreement, and does not 
        coincide with any special date as the end or start 
        of the employment contract.
        """
        expected_result = 0.0
        hiring_date = datetime.strptime('01/02/2022', '%d/%m/%Y')
        end_date = None
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5 # 50%
        calculate_range = False
        
        result = calculate_real_capacity(DEFAULT_AGREEMENT_HOURS, hiring_date, end_date, date, to_date, fte, calculate_range)

        self.assertEqual(result, expected_result)

    @freeze_time("2022-11-11")
    def test_calculate_real_capacity_return_only_work_days_when_employee_has_end_date(self):
        """ 
        A normal month is a definition for a month where the employee 
        will work 100% of the hours of his agreement, and does not 
        coincide with any special date as the end or start 
        of the employment contract.
        """
        expected_result = 33.5
        hiring_date = datetime.strptime('01/02/2021', '%d/%m/%Y')
        end_date = datetime.strptime('15/01/2022', '%d/%m/%Y')
        date = datetime.strptime('01/01/2022', '%d/%m/%Y')
        to_date = datetime.strptime('31/01/2022', '%d/%m/%Y')
        fte = 0.5 # 50%
        calculate_range = False
        
        result = calculate_real_capacity(DEFAULT_AGREEMENT_HOURS, hiring_date, end_date, date, to_date, fte, calculate_range)

        self.assertEqual(result, expected_result)



import unittest
from unittest.mock import Mock, patch
from datetime import datetime


class TestSuite(unittest.TestCase):
    def _create_GSheet_mock():
        service_mock = Mock()
        
        read_gsheet_mock = Mock()
        read_gsheet_mock.read_gsheet.return_value = {'values': [
            ['10/09/2022', '0'],
            ['11/09/2022', '0'],
            ['12/09/2022', '8'],
            ['13/09/2022', '8'],
            ['14/09/2022', '8'],
            ['15/09/2022', '8'],
            ['16/09/2022', '5.5'],
            ['17/09/2022', '0'],
            ['18/09/2022', '0'],
            ['19/09/2022', '8'],
        ]}
        read_gsheet_mock.get_sheetnames.return_value = {'sheets': [
            { 
                'properties': {
                    'title': 'Summary'
                }
            },
             { 
                'properties': {
                    'title': 'Sheet 1'
                }
            },
              { 
                'properties': {
                    'title': 'Sheet 2'
                }
            }
        ]}
        service_mock.return_value = read_gsheet_mock
        return service_mock

    @patch('gc_google_services_api.gsheet.GSheet', new=_create_GSheet_mock())
    def test_read_gsheet_should_call_google_api_with_credentials_and_correct_params(self):
        expected_result = {'totalHours': 45.5, 'days': {10: '0', 11: '0', 12: '8', 13: '8', 14: '8', 15: '8', 16: '5.5', 17: '0', 18: '0', 19: '8'}}
        spreadsheet_id = '111111'
        sheet_name = 'SHEET_NAME_TEST'
        month = 9
        year = 2022
        
        from ms_imputedhours_core.agreements import Agreements
        
        response = Agreements(spreadsheet_id).get_hours_by_month(month, year, sheet_name)

        self.assertEqual(response, expected_result)

    @patch('gc_google_services_api.gsheet.GSheet', new=_create_GSheet_mock())
    def test_get_hours_by_range_return_agreement_hours_by_range(self):
        expected_result = {'days': {'12-9': '8', '13-9': '8', '14-9': '8', '15-9': '8', '16-9': '5.5', '17-9': '0'}, 'totalHours': 37.5}
        spreadsheet_id = '111111'
        from_date = datetime.strptime('12/09/2022', '%d/%m/%Y')
        to_date = datetime.strptime('17/09/2022', '%d/%m/%Y')
        sheet_name = 'SHEET_NAME_TEST'
        
        from ms_imputedhours_core.agreements import Agreements
        
        response = Agreements(spreadsheet_id).get_hours_by_range(from_date, to_date, sheet_name)

        self.assertEqual(response, expected_result)

    @patch('gc_google_services_api.gsheet.GSheet', new=_create_GSheet_mock())
    def test_get_all_office_names_returns_all_sheet_names(self):
        expected_result = ['Sheet 1', 'Sheet 2']
        spreadsheet_id = '111111'
        
        from ms_imputedhours_core.agreements import Agreements
        
        response = Agreements(spreadsheet_id).get_all_office_names()

        self.assertEqual(response, expected_result)



import os
import base64
import json
from datetime import datetime
from gc_google_services_api.gsheet import GSheet

SCOPES = [
    'https://www.googleapis.com/auth/spreadsheets.readonly',
]
RANGE_NAME = os.getenv('GOOGLE_SHEET_AGREEMENT_RANGE', 'A2:B800')
CREDENTIALS_BASE64 = os.getenv('GOOGLE_SHEET_CREDENTIALS', '')
SHEET_NAMES_TO_IGNORE = ['Summary']

try:
    CREDENTIALS_CONTENT = json.loads(base64.b64decode(CREDENTIALS_BASE64))
except json.JSONDecodeError:
    CREDENTIALS_CONTENT = ''

service = GSheet(CREDENTIALS_CONTENT, SCOPES)

class Agreements(object):
    def __init__(self, spreadsheet_id) -> None:
        self.spreadsheet_id = spreadsheet_id

    def get_hours_by_month(self, month, year, sheet_name):
        try:
            values = service.read_gsheet(sheet_name, self.spreadsheet_id, RANGE_NAME)
        except Exception as e:
            print('[ERROR] - get_hours_by_month: ', e)
            values = {'values': []}

        day_hours_data = {
            'totalHours': 0.0,
            'days': {}
        }
        
        for day_data in values['values']:
            day_obj = datetime.strptime(day_data[0], '%d/%m/%Y')
    
            if day_obj.month == month and day_obj.year == year:
                day_hours = '{}'.format(day_data[1]).replace(',', '.')
                day_hours_data['totalHours'] += float(day_hours)
                day_hours_data['days'][day_obj.day] = day_hours

        return day_hours_data

    def get_hours_by_range(self, from_date, to_date, sheet_name):
        try:
            values = service.read_gsheet(sheet_name, self.spreadsheet_id, RANGE_NAME)
        except Exception as e:
            print('[ERROR] - get_hours_by_range: ', e)
            values = {'values': []}

        day_hours_data = {
            'totalHours': 0.0,
            'days': {}
        }

        for day_data in values['values']:
            day_obj = datetime.strptime(day_data[0], '%d/%m/%Y')
            is_date_after_from_date = (day_obj.month, day_obj.year) >= (from_date.month, from_date.year)
            is_date_before_to_date = (day_obj.month, day_obj.year) <= (to_date.month, to_date.year)

            if is_date_after_from_date and is_date_before_to_date:
                is_day_after_from_date = (day_obj.day, day_obj.month) >= (from_date.day, from_date.month)
                is_dat_before_to_date = (day_obj.day, day_obj.month) <= (to_date.day, to_date.month)

                if is_day_after_from_date and is_dat_before_to_date:
                    day_key = '{}-{}'.format(day_obj.day,day_obj.month)
                    day_hours = '{}'.format(day_data[1]).replace(',', '.')
                    day_hours_data['totalHours'] += float(day_hours)
                    day_hours_data['days'][day_key] = day_hours

        return day_hours_data

    def get_all_office_names(self):
        try:
            values = service.get_sheetnames(self.spreadsheet_id)
        except Exception as e:
            print('[ERROR] - get_all_agreements: ', e)
            values = {}
        
        sheet_names = []
        for sheet in values.get('sheets', []):
            sheet_name = sheet['properties']['title']
            if sheet_name not in SHEET_NAMES_TO_IGNORE:
                sheet_names.append(sheet_name)

        return sheet_names



from datetime import datetime

HUNDRED_PERCENT = 100

def get_real_fte(successfactor_data, employeeFTE, date):
    today = datetime.today()
    is_date_for_future = (date.year, date.month) >= (today.year, today.month)
    fte = successfactor_data['FTE']

    if employeeFTE and is_date_for_future:
        fte = employeeFTE['fte']

    return fte / HUNDRED_PERCENT if fte > 0 else fte

def calculate_real_capacity(agreement_hours, hiringdate, end_date, date, to_date, fte, calculate_range=False):
    def _is_date_before_hired():
        return hiringdate and ((date.year, date.month) < (hiringdate.year, hiringdate.month))

    def _is_date_same_hired():
        return hiringdate and ((date.year, date.month) == (hiringdate.year, hiringdate.month))

    def _is_date_between_hired_and_end_date():
        is_date_before_end_date = not end_date or (
            (date.year, date.month) < (end_date.year, end_date.month))
        return is_date_before_end_date and hiringdate and ((date.year, date.month) > (hiringdate.year, hiringdate.month))

    def _is_date_same_end_date():
        return end_date and ((date.year, date.month) == (end_date.year, end_date.month))

    def _is_date_after_end_date():
        return end_date and (date.year, date.month) > (end_date.year, end_date.month)

    real_capacity = agreement_hours.get('totalHours', 0)

    if _is_date_before_hired():
        # Months prior to hiring the employee
        real_capacity = 0
    elif _is_date_same_hired():
        # Same month of hiring the employee
        real_capacity = 0
        days = agreement_hours.get('days', {})

        for day, hours in days.items():
            if (int(day) >= hiringdate.day):
                real_capacity += float(hours.replace(',', '.'))
    elif calculate_range and _is_date_between_hired_and_end_date():
        # Just when we want to calculate hours by range of date
        real_capacity = 0
        days = agreement_hours.get('days', {})
        for dayMonth, hours in days.items():
            day, month = dayMonth.split('-')
            is_date_after_from = (int(day), int(
                month)) >= (date.day, date.month)
            is_date_before_to = (int(day), int(month)) <= (
                to_date.day, to_date.month)
            if is_date_after_from and is_date_before_to:
                real_capacity += float(hours.replace(',', '.'))

    if _is_date_same_end_date():
        # Same month of the termination of the employment contract
        real_capacity = 0
        days = agreement_hours.get('days', {})
        for day, hours in days.items():
            if (int(day) <= end_date.day):
                real_capacity += float(hours.replace(',', '.'))

    elif _is_date_after_end_date():
        # Months after the termination of the employee's contract
        real_capacity = 0

    return real_capacity * fte


from setuptools import setup, find_packages
from ms_imputedhours_core import __version__
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / 'README.md').read_text()

setup(
      name='ms_imputedhours_core',
      version=__version__,
      packages=find_packages(),
      author='Jonathan Rodriguez Alejos',
      author_email='jrodriguez.5716@gmail.com',
      install_requires=open('requirements.txt').read().splitlines(),
      long_description=long_description,
      long_description_content_type='text/markdown'
)